package lightcontainer.domains;

import lightcontainer.domains.client.Context;
import lightcontainer.enumerations.TaskStatus;
import lightcontainer.enumerations.TaskType;
import lightcontainer.protocol.ProtocolWriter;

public class Task {
    // Variables
    private TaskStatus status;
    private ProtocolWriter.ProtocolResult command;
    private String storeDomain;

    /**
     * Défini le context courrant dans laquelle la tâche opère
     */
    private Context context;

    public Task(Context context, TaskStatus status, ProtocolWriter.ProtocolResult command) {
        this.context = context;
        this.status = status;
        this.command = command;
    }

    public static Task newInstance(Context context, ProtocolWriter.ProtocolResult command) {
        Task task = new Task(context, TaskStatus.PENDING, command);
        return task;
    }

    /**
     * Permet de savoir si la réponse est destinée au client
     * @param storeDomain Nom du store back end fournissant la réponse.
     * @return TRUE si le client doit recevoir cette réponse.
     */
    public boolean isResponseOfClient(String storeDomain) {
        return (status == TaskStatus.PROCESSING && this.storeDomain.equals(storeDomain));
    }

    /**
     * Permet de récupérer le login du client associé à la tâche
     * @return Login du client
     */
    public String getClient() {
        return context.getLogin();
    }

    public ProtocolWriter.ProtocolResult getCommand() {
        return this.command;
    }

    public void setDomain(String storeDomain) {
        this.storeDomain = storeDomain;
        if (storeDomain != null) {
            this.status = TaskStatus.PROCESSING;
        }
    }

    /**
     * Défini le context courrant dans laquelle la tâche opère
     */
    public Context getContext() {
        return this.context;
    }
}
