package lightcontainer.protocol.rules.reader;

import lightcontainer.domains.client.Context;
import lightcontainer.interfaces.ProtocolRepository;
import lightcontainer.protocol.ProtocolReader;
import lightcontainer.protocol.rules.writer.SignErrorRule;
import lightcontainer.protocol.rules.writer.SignOkRule;

public class SignupRule extends ProtocolReader {
    // Constants
    private static final String PATTERN = "^SIGNUP ([A-Za-z0-9]{5,20}) ([^ !]{5,50})\r\n$";
    private static final String NAME = "SIGNUP";
    private static final int LOGIN = 0;
    private static final int PASSWORD = 1;

    private ProtocolRepository protocolRep;

    public SignupRule(ProtocolRepository protocolRep) {
        super(NAME, PATTERN);
        this.protocolRep = protocolRep;
    }

    public class Result extends ProtocolResult {
        //Variables
        private final String login;
        private final String password;

        public Result(Context context, String login, String password) {
            super(context);
            this.login = login;
            this.password = password;
        }

        public String getLogin() {
            return login;
        }

        public String getPassword() {
            return password;
        }


    }

    @Override
    protected SignupRule.Result onExecuted(Context context, String... data) {
        SignupRule.Result result = new SignupRule.Result(context, data[LOGIN], data[PASSWORD]);

        if (context.createUser(result.getLogin(), result.getPassword())) {
            result.setResultCommand(this.protocolRep.executeWriter(SignOkRule.NAME), ResultCmdReceiver.CLIENT);
        } else {
            result.setResultCommand(this.protocolRep.executeWriter(SignErrorRule.NAME), ResultCmdReceiver.CLIENT);
        }

        return null;
    }

    @Override
    protected ProtocolReader.ProtocolResult onError(Context context) {
        ProtocolReader.ProtocolResult result = new ProtocolReader.ProtocolResult(context);

        result.setResultCommand(protocolRep.executeWriter(SignErrorRule.NAME), ResultCmdReceiver.CLIENT);
        return result;
    }
}
