package lightcontainer.storage;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;

public class Repository {

    private final String filePath;
    private final Adapter adapter;

    /**
     * @param filePath The path to the configuration file
     * @param adapter  The adapter that converts objects to string and vice-versa
     */
    public Repository(String filePath, Adapter adapter) {
        this.filePath = filePath;
        this.adapter = adapter;
    }

    /**
     * Saves configuration file
     */
    public void save() {
        if (filePath != null) {
            String jsonAppData = adapter.toString();
            try (BufferedWriter bufferedWriter = Files.newBufferedWriter(Paths.get(filePath).toAbsolutePath(), StandardCharsets.UTF_8, StandardOpenOption.CREATE, StandardOpenOption.TRUNCATE_EXISTING)) {
                bufferedWriter.write(jsonAppData);
                bufferedWriter.flush();
            } catch (IOException e) {
                System.out.println("Error while saving configuration file !");
            }
        }
    }


    /**
     * Loads configuration file
     *
     * @return AppData
     */
    public AppData load() {
        String jsonString = readFile(filePath);
        return adapter.fromString(jsonString);
    }

    private String readFile(String filePath) {
        StringBuilder builder = new StringBuilder();
        try (BufferedReader reader = Files.newBufferedReader(Paths.get(filePath).toAbsolutePath(), StandardCharsets.UTF_8)) {
            while (reader.ready()) {
                builder.append(reader.readLine());
            }
        } catch (IOException e) {
            System.out.println("Error while reading configuration file");
            builder.setLength(0);
        }
        return builder.toString();
    }
}
